
#define ZF_LOW    0xEF
#define ZF_DR     0x00
#define ZF_ZIFR   0xC0
#define ZF_ZOFR   0xC1
#define ZF_RIFR   0xC2
#define ZF_ROFR   0xC3
#define ZF_IMR    0xC4
#define ZF_ISR    0xC4
#define ZF_ZIBTR  0xC5
#define ZF_ZITOR  0xC6
#define ZF_CR     0xC7
#define ZF_ER     0xC7
#define ZF_RIBTR  0xC8
#define ZF_RITOR  0xC9

#define ZF_IMR_ZIBT  0x01
#define ZF_IMR_ZITO  0x02
#define ZF_IMR_RIBT  0x04
#define ZF_IMR_RITO  0x08

enum
{
  ZF_ERR_OK,
  ZF_ERR_OVF
};

u8 zf_rd(u8 a) __naked
{
  a;  // to avoid SDCC warning
  __asm
    ld hl, #2
    add hl, sp
    ld b, (hl)
    ld c, #ZF_LOW
    in l, (c)
    ret
  __endasm;
}

void zf_wr(u8 a, u8 v) __naked
{
  a;  // to avoid SDCC warning
  v;  // to avoid SDCC warning
  __asm
    ld hl, #2
    add hl, sp
    ld b, (hl)
    inc hl
    ld c, #ZF_LOW
    ld a, (hl)
    out (c), a
    ret
  __endasm;
}

void zf_wait(u32 del)  // !!!
{
  volatile u32 a = del;
  while (a--);
}

void print(u8 *p) // !!!
{
  u8 c;
  while (c = *p++)
    putchar(c);
}

void dump(u8 *p) // !!!
{
  u8 xx, yy;

  for (yy = 0; yy < 36; yy++)
  {
    if (pcy >= 36) break;

    pca = 12;
    printf("\r\n%04X  ", (int)p);

    for (xx = 6; xx < (6 + 16 * 3); xx += 3)
    {
      pca = 9;
      printf("%02X ", *p++);
    }
  }
}

void zf_rd_fifo(u8 *buf, u8 c)
{
  c, buf;    // to avoid SDCC warning
  
  __asm
    ld hl, #2
    add hl, sp
    ld e, (hl)
    inc hl
    ld d, (hl)
    inc hl
    ld b, (hl)
    ld c, #ZF_LOW
    ex de, hl
    inir
    ret
  __endasm;
}

void zf_rd_fifo_dma(u8 *buf, u8 c)
{
  ts_wreg(TS_DMAWPA, ZF_DR);
  ts_wreg(TS_DMAWPD, TS_WPD_COM);
  ts_set_dma_daddr_p((u16)buf, 0xF6);
  // ts_set_dma_daddr_p(0xC000, 0xF6);
  ts_set_dma_size(c, 1);
  ts_dma_start(TS_DMA_WTP_RAM);
  ts_dma_wait();
}

u8 zf_at_raw(u8 *buf)
{
  u8 c;

  printf("\aD%s", buf);  // !!!

  while (c = *buf++)
  {
    if (!zf_rd(ZF_ZOFR))
      return ZF_ERR_OVF;
    else
      zf_wr(ZF_DR, c);
  }

  return ZF_ERR_OK;
}

void zf_resp() // !!!
{
  u8 c;
  u8 bf[256];

  while (!zf_rd(ZF_ZIFR));

  while (c = zf_rd(ZF_ZIFR))
  {
    zf_rd_fifo(bf, c);
    // printf("\aEIFR: \aC%d\r\n", c);
    bf[c] = 0;
    printf("\aF%s", bf);
  }
}


void zf_resp2() // !!!
{
  u8 *bf = (u8*)0xD000;
  u8 c;
  u16 len = 0;

  printf("\a7");
  while (!zf_rd(ZF_ZIFR));
  zf_wait(500);

  while (c = zf_rd(ZF_ZIFR))
  {
    u8 c1 = c & 0xFE;
    if (!c1) break;
    
    // zf_rd_fifo(&bf[len], c1);
    zf_rd_fifo_dma(&bf[len], c1);
    printf("\aE%d \a7", c1);
    len += c1;
  }
  
  if (c)
  {
    zf_rd_fifo(&bf[len], c);
    printf("\aC%d", c);
    len += c;
  }
  
  printf("\r\n\aB%d\r\n\a7", len);
  bf[len] = 0;
  print(bf);
}

void zf_resp3() // !!!
{
  u8 *bf = (u8*)0xC000;
  u8 c;
  u16 len = 0;

  printf("\a7");
  while (!zf_rd(ZF_ZIFR));
  zf_wait(500);

  while (c = zf_rd(ZF_ZIFR))
  {
    u8 c1 = c & 0xFE;
    if (!c1) break;
    
    zf_rd_fifo_dma(bf, c1);
    printf("\aE%d \a7", c1);
    len += c1;
  }
  
  if (c)
  {
    zf_rd_fifo(bf, c);
    printf("\aC%d", c);
    len += c;
  }
  
  printf("\r\n\aB%d\r\n\a7", len);
}

